#!/bin/sh -e

PROG="${0##*/}"
MOZ_APPNAME='firefox'
MOZ_WORKDIR="$(mktemp -dt $MOZ_APPNAME.XXXXXXXXXX)"
MOZ_XPIFILE="$(mktemp "$MOZ_WORKDIR/xpi.XXXXXXXXXX")"

error() {
    printf 'Error: %s\n' "$@" >&2
    exit 1
}

exit_handler() {
    local rc=$?
    trap - EXIT
    [ -z "$MOZ_WORKDIR" ] || rm -rf -- "$MOZ_WORKDIR"
    exit $rc
}

trap exit_handler EXIT HUP INT PIPE TERM QUIT

firefox_run="$1" && shift || error "first argument required"
firefox_bin="$1" && shift || error "second argument required"
xpidir="$1"	 && shift || error "third argument required"

[ -x "$firefox_run" ] || error "not executable: $firefox_run"
[ -x "$firefox_bin" ] || error "not executable: $firefox_bin"
[ -d "$xpidir" ] || error "not directory: $xpidir"

# We must _always_ fake HOME variable. 
export HOME="$MOZ_WORKDIR"

pushd "$xpidir" > /dev/null
zip -qr - . > "$MOZ_XPIFILE" || error "unable to form extension packet"
popd > /dev/null
"$firefox_run" "$firefox_bin" -nox -install-global-extension "$MOZ_XPIFILE" 2>/dev/null >/dev/null || 
    error "install global extension faild"
